package schedframe.scheduling;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import schedframe.resources.computing.ComputingResource;
import schedframe.resources.units.ProcessingElements;
import schedframe.resources.units.ResourceUnit;
import schedframe.resources.units.ResourceUnitName;
import schedframe.resources.units.StandardResourceUnitName;

public class ResourceItem {

	protected Map<ResourceUnitName, ResourceUnit> usedResources;
	protected Set<String> resourceNames;

	public ResourceItem(Map<ResourceUnitName, ResourceUnit> usedResources){
		this.usedResources = usedResources;
		this.resourceNames = saveResourceNames();
	}
	
	public Map<ResourceUnitName, ResourceUnit> getResourceUnits(){
		return usedResources;
	}
	
	public Set<String> getResourceNames(){
		return resourceNames;
	}
	
	private Set<String> saveResourceNames(){
		Set<String> resourceNames;
		ProcessingElements pes = (ProcessingElements) usedResources.get(StandardResourceUnitName.PE);
		resourceNames = new HashSet<String>(pes.size(), 1);
		for (ComputingResource resource: pes) {

			LinkedList<ComputingResource> toExamine = new LinkedList<ComputingResource>();
			toExamine.push(resource);

			while (!toExamine.isEmpty()) {
				ComputingResource compResource = toExamine.pop();
				List<ComputingResource> resources = compResource.getChildren();
				if(resources.isEmpty()){
					if(!resourceNames.contains(compResource.getFullName())){
						resourceNames.add(compResource.getFullName());
					}
				} else {
					for (int i = 0; i < resources.size(); i++) {
						ComputingResource resourceChild = resources.get(i);
						toExamine.addLast(resourceChild);
					}
				}
			}
		}
		return resourceNames;
	}
}
